//+------------------------------------------------------------------+
//|                                          OutsideBar_AlexSTAL.mq4 |
//|                                       Copyright  2011, AlexSTAL |
//|                                           http://www.alexstal.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, AlexSTAL"
#property link      "http://www.alexstal.ru"

//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
//|     High Low          |
//|                                                                  |
//| :                                                       |
//|   symbol            - c                   |
//|   timeframe         -                           |
//|   shift             -                                  |
//|   UseSmallerTFforEB -           |
//|                          (     |
//|                          )                  |
//|                                                                  |
//|  :                                           |
//|  1:  High,  Low                                      |
//| -1:  Low,  High                                      |
//|  0: High = Low                                                   |
//+------------------------------------------------------------------+ 
int GetOrderFormationBarHighLow(string symbol, int timeframe, int shift, bool UseSmallerTFforEB)
{
   int tmp = 0;
   // High == Low
   if ( iHigh(symbol, timeframe, shift) == iLow(symbol, timeframe, shift) )
      return(tmp);
   
   //    
   //       
   if ( UseSmallerTFforEB )
     {
      //        )))
      for (int i = 1; i <= NumberOfPassesSearch(timeframe); i++)
        {
         tmp = SmallTFLogicBarHighLow(symbol, timeframe, shift, SmallTFforNumberOfPassesSearch(timeframe, i));
         if ( (tmp == 1) || (tmp == -1) )
            break;
        }
      if ( (tmp != 1) && (tmp != -1) )
         tmp = SimpleLogicBarHighLow(symbol, timeframe, shift);
     } else {
      tmp = SimpleLogicBarHighLow(symbol, timeframe, shift);
     }
   return(tmp);
}

//+------------------------------------------------------------------+
//|     High Low          |
//|                                       |
//|                                                                  |
//| :                                                       |
//|   symbol     - c                          |
//|   timeframe  -                                  |
//|   shift      -                                         |
//|   TFsmall    -                  |
//|                                                                  |
//|  :                                           |
//|  0:                                                  |
//|  1:  High,  Low                                      |
//| -1:  Low,  High                                      |
//|  2: High = Low                                                   |
//| -2:   - High  Low     |
//+------------------------------------------------------------------+
int SmallTFLogicBarHighLow(string symbol, int timeframe, int shift, int TFsmall)
{
   double BarH = iHigh(symbol, timeframe, shift);     // High 
   double BarL = iLow(symbol, timeframe, shift);      // Low 
   if ( BarH == BarL )
      return(2);
   datetime BarDTs = iTime(symbol, timeframe, shift); //  
   datetime BarDTe;                                   //  
   if ( timeframe != PERIOD_MN1 )
      BarDTe = BarDTs + timeframe*60 - 1;         
   else
      //       
      BarDTe = BarDTs + 1440*NumberDaysMonth(BarDTs)*60 - 1;         //  
   
   //  
   datetime tmpDT;
   
   //    
   int indBarS = iBarShift(symbol, TFsmall, BarDTs, false);
   if ( indBarS < 0 )
      return(0);
   tmpDT = iTime(symbol, TFsmall, indBarS);
   if ( tmpDT < BarDTs )
      indBarS = indBarS - 1;
      
   //    
   int indBarE = iBarShift(symbol, TFsmall, BarDTe, false);
   if ( indBarE < 0 )
      return(0);
   
   //   
   //      (  )
   int tmpI = 0;
   for (int i = indBarS; i >= indBarE; i--)
     {
      if ( iHigh(symbol, TFsmall, i) >= BarH )
         tmpI = tmpI + 1;
      if ( iLow(symbol, TFsmall, i) <= BarL )
         tmpI = tmpI + 2;
      if ( tmpI != 0 )
         break;
     }
   switch(tmpI)
     {
      case 0:
         return(0);  //   ( ,   )
         break;
      case 1:
         return(1);  //  High
         break;
      case 2:
         return(-1); //  Low
         break;
      case 3:
         return(-2); //   - High  Low     
         break;
      default:
         return(0);
         break;
     }
}

//+------------------------------------------------------------------+
//|            |
//| High/Low                                                         |
//+------------------------------------------------------------------+
//|     !!!                        |
//+------------------------------------------------------------------+
//| :                                                       |
//|   symbol     - c                          |
//|   timeframe  -                                  |
//|   shift      -                                         |
//|                                                                  |
//|  :                                           |
//|  1:  High,  Low                                      |
//| -1:  Low,  High                                      |
//|  0: High = Low                                                   |
//+------------------------------------------------------------------+
int SimpleLogicBarHighLow(string symbol, int timeframe, int shift)
{
   //    Open / Close
   int res = 0;
   if ( iHigh(symbol, timeframe, shift) == iLow(symbol, timeframe, shift) )
      return(res);
   
   if ( iClose(symbol, timeframe, shift) > iOpen(symbol, timeframe, shift) )
      res = -1;
   if ( iClose(symbol, timeframe, shift) < iOpen(symbol, timeframe, shift) )
      res = 1;
   
   if (res == 0) //  Close = Open
     {
      double a1 = iHigh(symbol, timeframe, shift) - iClose(symbol, timeframe, shift);
      double a2 = iClose(symbol, timeframe, shift) - iLow(symbol, timeframe, shift);
      if ( a1 > a2 )
         res = -1;
      if ( a1 < a2 )
         res = 1;
      if (res == 0)  //     ))))
         res = 1;    // :) -   !  !
     }
   return(res);
}

//+------------------------------------------------------------------+
//|             |
//|     High/Low                     |
//+------------------------------------------------------------------+
int NumberOfPassesSearch(int timeframe)
{
   switch(timeframe)
     {
      case     1: return(0); break;
      case     5: return(1); break;
      case    15: return(1); break;
      case    30: return(2); break;
      case    60: return(2); break;
      case   240: return(2); break;
      case  1440: return(3); break;
      case 10080: return(4); break;
      case 43200: return(4); break;
      default: return(0); break;
     }
}

//+------------------------------------------------------------------+
//|                      |
//|     NumberOfPassesSearch   |
//+------------------------------------------------------------------+
int SmallTFforNumberOfPassesSearch(int timeframe, int Passes)
{
   switch(timeframe)
     {
      case     5:
         switch(Passes)
           {
            case 1: return(1); break;
            default: return(0); break;
           }
         break;
      case    15:
         switch(Passes)
           {
            case 1: return(1); break;
            default: return(0); break;
           }
         break;
      case    30:
         switch(Passes)
           {
            case 1: return(5); break;
            case 2: return(1); break;
            default: return(0); break;
           }
         break;
      case    60:
         switch(Passes)
           {
            case 1: return(5); break;
            case 2: return(1); break;
            default: return(0); break;
           }
         break;
      case   240:
         switch(Passes)
           {
            case 1: return(30); break;
            case 2: return(1); break;
            default: return(0); break;
           }
         break;
      case  1440:
         switch(Passes)
           {
            case 1: return(240); break;
            case 2: return(30); break;
            case 3: return(1); break;
            default: return(0); break;
           }
         break;
      case 10080:
         switch(Passes)
           {
            case 1: return(1440); break;
            case 2: return(240); break;
            case 3: return(30); break;
            case 4: return(1); break;
            default: return(0); break;
           }
         break;
      case 43200:
         switch(Passes)
           {
            case 1: return(1440); break;
            case 2: return(240); break;
            case 3: return(30); break;
            case 4: return(1); break;
            default: return(0); break;
           }
         break;
      default: return(0); break;
     }
}

//+------------------------------------------------------------------+
//|                                   |
//+------------------------------------------------------------------+
int NumberDaysMonth(datetime Date)
{
   switch(TimeMonth(Date))
     {
      case  1: return(31); break;
      case  2:
         if ( (TimeYear(Date) % 4) == 0 )
            return(29);
         else
            return(28);
         break;
      case  3: return(31); break;
      case  4: return(30); break;
      case  5: return(31); break;
      case  6: return(30); break;
      case  7: return(31); break;
      case  8: return(31); break;
      case  9: return(30); break;
      case 10: return(31); break;
      case 11: return(30); break;
      case 12: return(31); break;
      default: return(0); break;
     }
}